from mgraph_ai_service_cache.schemas.cache.enums.Enum__Cache__Store__Strategy import Enum__Cache__Store__Strategy

DEFAULT_CACHE__STORE__STRATEGY     = Enum__Cache__Store__Strategy.TEMPORAL
DEFAULT_CACHE__NAMESPACE          = 'default'

DEFAULT__HTTP_CODE__FILE_NOT_FOUND = 404